@@ -1,40 +1,34 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
-
-# load python 3, fallback to python 2 if it fails
-try:
-    from urllib.parse import unquote, unquote_plus, quote_plus
-except ImportError:
-    from urllib import unquote, unquote_plus, quote_plus  # type: ignore
+from urllib.parse import unquote, unquote_plus, quote_plus
 from datetime import datetime, timedelta
 from itertools import tee
-import sys
+from distutils.util import strtobool
 from flask import (
     render_template, abort, url_for,
     Response, stream_with_context, request, session, jsonify
 )
 
 import logging
+import json
 
 from pypuppetdb.QueryBuilder import (ExtractOperator, AndOperator,
                                      EqualsOperator, FunctionOperator,
                                      NullOperator, OrOperator,
-                                     LessEqualOperator, RegexOperator)
+                                     LessEqualOperator, RegexOperator,
+                                     GreaterEqualOperator)
 
 from puppetboard.forms import ENABLED_QUERY_ENDPOINTS, QueryForm
 from puppetboard.utils import (get_or_abort, yield_or_stop,
-                               get_db_version)
+                               get_db_version, is_bool)
 from puppetboard.dailychart import get_daily_reports_chart
 
-try:
-    import CommonMark as commonmark
-except ImportError:
-    import commonmark
+import commonmark
 
 from puppetboard.core import get_app, get_puppetdb, environments
 
-from . import __version__
+from puppetboard.version import __version__
 
 REPORTS_COLUMNS = [
     {'attr': 'end', 'filter': 'end_time',
@@ -63,6 +57,28 @@ log = logging.getLogger(__name__)
 puppetdb = get_puppetdb()
 
 
+menu_entries = [
+    ('index', 'Overview'),
+    ('nodes', 'Nodes'),
+    ('facts', 'Facts'),
+    ('reports', 'Reports'),
+    ('metrics', 'Metrics'),
+    ('inventory', 'Inventory'),
+    ('catalogs', 'Catalogs'),
+    ('radiator', 'Radiator'),
+    ('query', 'Query')
+]
+
+if not app.config.get('ENABLE_QUERY'):
+    menu_entries.remove(('query', 'Query'))
+
+if not app.config.get('ENABLE_CATALOG'):
+    menu_entries.remove(('catalogs', 'Catalogs'))
+
+
+app.jinja_env.globals.update(menu_entries=menu_entries)
+
+
 @app.template_global()
 def version():
     return __version__
@@ -85,9 +101,10 @@ def metric_params(db_version):
     query_type = ''
 
     # Puppet Server is enforcing new metrics API (v2)
-    # starting with versions 6.9.1 and 5.3.12
+    # starting with versions 6.9.1, 5.3.12, and 5.2.13
     if (db_version > (6, 9, 0) or
-            (db_version > (5, 3, 11) and db_version < (6, 0, 0))):
+            (db_version > (5, 3, 11) and db_version < (6, 0, 0)) or
+            (db_version > (5, 2, 12) and db_version < (5, 3, 10))):
         metric_version = 'v2'
     else:
         metric_version = 'v1'
@@ -140,11 +157,7 @@ def index(env):
             puppetdb.metric,
             "{0}{1}".format(prefix, ':%sname=num-resources' % query_type),
             version=metric_version)
-        avg_resources_node = get_or_abort(
-            puppetdb.metric,
-            "{0}{1}".format(prefix,
-                            ':%sname=avg-resources-per-node' % query_type),
-            version=metric_version)
+
         metrics['num_nodes'] = num_nodes['Value']
         metrics['num_resources'] = num_resources['Value']
         try:
@@ -446,6 +459,7 @@ def reports_ajax(env, node_name):
     order_dir = request.args.get('order[0][dir]', 'desc')
     order_args = '[{"field": "%s", "order": "%s"}]' % (order_filter, order_dir)
     status_args = request.args.get('columns[1][search][value]', '').split('|')
+    date_args = request.args.get('columns[0][search][value]', '')
     max_col = len(REPORTS_COLUMNS)
     for i in range(len(REPORTS_COLUMNS)):
         if request.args.get("columns[%s][data]" % i, None):
@@ -469,6 +483,20 @@ def reports_ajax(env, node_name):
             "configuration_version", r"%s" % search_arg))
         reports_query.add(search_query)
 
+    if date_args:
+        dates = json.loads(date_args)
+
+        if len(dates) > 0:
+            date_query = AndOperator()
+
+            if 'min' in dates:
+                date_query.add(GreaterEqualOperator('end_time', dates['min']))
+
+            if 'max' in dates:
+                date_query.add(LessEqualOperator('end_time', dates['max']))
+
+            reports_query.add(date_query)
+
     status_query = OrOperator()
     for status_arg in status_args:
         if status_arg in ['failed', 'changed', 'unchanged']:
@@ -598,32 +626,50 @@ def facts(env):
     """
     envs = environments()
     check_env(env, envs)
-    facts = []
-    order_by = '[{"field": "name", "order": "asc"}]'
     facts = get_or_abort(puppetdb.fact_names)
 
-    facts_columns = [[]]
-    letter = None
-    letter_list = None
-    break_size = (len(facts) / 4) + 1
+    # we consider a column label to count for ~5 lines
+    column_label_height = 5
+
+    # 1 label per different letter and up to 3 more labels for letters spanning
+    # multiple columns.
+    column_label_count = 3 + len(set(map(lambda fact: fact[0].upper(), facts)))
+
+    break_size = (len(facts) + column_label_count * column_label_height) / 4.0
     next_break = break_size
+
+    facts_columns = []
+    facts_current_column = []
+    facts_current_letter = []
+    letter = None
     count = 0
+
     for fact in facts:
         count += 1
 
-        if letter != fact[0].upper() or not letter:
-            if count > next_break:
-                # Create a new column
-                facts_columns.append([])
-                next_break += break_size
-            if letter_list:
-                facts_columns[-1].append(letter_list)
-            # Reset
-            letter = fact[0].upper()
-            letter_list = []
+        if count > next_break:
+            next_break += break_size
+            if facts_current_letter:
+                facts_current_column.append(facts_current_letter)
+            if facts_current_column:
+                facts_columns.append(facts_current_column)
+            facts_current_column = []
+            facts_current_letter = []
+            letter = None
 
-        letter_list.append(fact)
-    facts_columns[-1].append(letter_list)
+        if letter != fact[0].upper():
+            if facts_current_letter:
+                facts_current_column.append(facts_current_letter)
+                facts_current_letter = []
+            letter = fact[0].upper()
+            count += column_label_height
+
+        facts_current_letter.append(fact)
+
+    if facts_current_letter:
+        facts_current_column.append(facts_current_letter)
+    if facts_current_column:
+        facts_columns.append(facts_current_column)
 
     return render_template('facts.html',
                            facts_columns=facts_columns,
@@ -720,7 +766,10 @@ def fact_ajax(env, node, fact, value):
         value = int(value)
     except ValueError:
         if value is not None and query is not None:
-            query.add(EqualsOperator('value', unquote_plus(value)))
+            if is_bool(value):
+                query.add(EqualsOperator('value', bool(strtobool(value))))
+            else:
+                query.add(EqualsOperator('value', unquote_plus(value)))
     except TypeError:
         pass
 
@@ -872,7 +921,7 @@ def metrics(env):
                 metrics.append(domain + ':' + prop)
     else:
         raise ValueError("Unknown metric version {} for database version {}"
-                         .format(metric_version, database_version))
+                         .format(metric_version, db_version))
 
     return render_template('metrics.html',
                            metrics=sorted(metrics),
