@@ -93,7 +93,7 @@ def client():
 
 
 def test_first_test():
-    assert app is not None, ("%s" % reg.app)
+    assert app is not None
 
 
 def test_no_env(client, mock_puppetdb_environments):
@@ -321,7 +321,7 @@ def test_offline_mode(client, mocker,
     for link in soup.find_all('link'):
         assert "//" not in link['href']
         if 'offline' in link['href']:
-            static_rv = client.get(link['href'])
+            rv = client.get(link['href'])
             assert rv.status_code == 200
 
     for script in soup.find_all('script'):
@@ -751,6 +751,27 @@ def test_facts_view(client, mocker, mock_puppetdb_environments):
     assert len(vals) == 4
 
 
+def test_facts_view_empty_when_no_facts(client,
+                                        mocker,
+                                        mock_puppetdb_environments):
+    query_data = {
+        'fact-names': [[]]
+    }
+
+    dbquery = MockDbQuery(query_data)
+
+    mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
+
+    rv = client.get('/facts')
+    assert rv.status_code == 200
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+    searchable = soup.find('div', {'class': 'searchable'})
+    vals = searchable.find_all('div', {'class': 'column'})
+    assert len(vals) == 0
+
+
 def test_fact_view_with_graph(client, mocker,
                               mock_puppetdb_environments,
                               mock_puppetdb_default_nodes):
@@ -953,6 +974,16 @@ def test_health_status(client):
     assert rv.data.decode('utf-8') == 'OK'
 
 
+def test_metric_params_5213():
+    query_type, metric_version = app.metric_params((5, 2, 13))
+    assert query_type == ''
+    assert metric_version == 'v2'
+
+    query_type, metric_version = app.metric_params((5, 2, 12))
+    assert query_type == ''
+    assert metric_version == 'v1'
+
+
 def test_metric_params_5312():
     query_type, metric_version = app.metric_params((5, 3, 12))
     assert query_type == ''
